//+---------------------------------------------------------------------+
//| Alpha_EAv01.mq4                                                     |
//| This system is designed for intra-day traders,on 30 Min timeframe.  |
//+---------------------------------------------------------------------+
#property copyright "Bassam Maamoon  2010"
#property link      "bmaamoon@yahoo.com"

#define  SLIPPAGE             	3
#define  COMMENTARY_LONG      	"MACD Buy"
#define  COMMENTARY_SHORT     	"MACD Sell"
//-------External Variables--------------------------------------- 1 --
extern bool five_digs_broker =false;// If the broker is 5 digits broker
extern double StopLoss   =30;       // SL for an opened order 
extern double Lots       =0.0;      // Strictly set amount of lots
extern double Prots      =0.15;     // Percent of free margin

//-------Global Variables
bool Work =true,                    // EA will work.
     Ans  =false,                   // Server response after closing
     Cls_B=false,                   // Criterion for closing Buy
     Cls_S=false,                   // Criterion for closing Sell
     Opn_B=false,                   // Criterion for opening Buy
     Opn_S=false;                   // Criterion for opening sell

string Symb;                        // Security name
datetime	CurrentBar;                // The bar that just opened
//--------------------------------------------------------------- 2 --
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
 {
   CurrentBar = Time[0];
   Symb=Symbol();                   // Security name    
  
  if (five_digs_broker ==true)
    StopLoss = StopLoss*10;
  return(0);
   }

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
{
    //----

    //----
    return(0);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+

int start()
  {
   int
   Total,                           // Amount of orders in a window    
   Tip=-1,                          // Type of selected order (B=0,S=1)
   Ticket,                          // Order number
   Cur_Hour=Hour();                 // The current hour time
   double
   Lot,                             // Amount of lots in a selected order 
   Lts,                             // Amount of lots in an opened order
   Min_Lot,                         // Minimal amount of lots   
   Step,                            // Step of lot size change   
   Free,                            // Current free margin
   Balance,                         // Currunt amount of money on the account
   One_Lot,                         // Price of one lot   
   OPrice,                          // Open Price of a selected order   
   SL,                              // SL of a selected order
   Cur_Min =Minute(),               // The current minute time
   Cur_time=Cur_Hour + Cur_Min/100; // The current time
//--------------------------------------------------------------- 3 --
   // Preliminary processing

   if(Work==false)                              // Critical error
     {
      Alert("for ",Symb,". Critical error. EA doesnt work.");
      return;                                   // Exit start()
     }
//--------------------------------------------------------------- 4 --
   // Orders accounting
  
   Total=0;                                     // Amount of orders
      for(int i=1; i<=OrdersTotal(); i++)          // Loop through orders
     {
      if (OrderSelect(i-1,SELECT_BY_POS)==true) // If there is the next one
        {                                       // Analyzing orders:
         if (OrderSymbol()!=Symb)continue;      // Another security
         if (OrderType()>1)                     // Pending order found
           {
            Alert("for ",Symb,". Pending order detected. EA doesnt work.");
            return;                             // Exit start()
           }
         Total++;                               // Counter of market orders
         if (Total>1)                           // No more than one order
           {
            Alert("for ",Symb,". Several market orders. EA doesnt work");
            return;                             //Exit start()
           }
         Ticket=OrderTicket();                  // Number of selected order     
         OPrice=OrderOpenPrice();               // Open Price of a selected order
         Tip   =OrderType();                    // Type of selected order
         SL    =OrderStopLoss();                // SL of selected order
         Lot   =OrderLots();                    // Amount of lots
        }
     }
//--------------------------------------------------------------- 5 --
   // Trading criteria

	if(NewBar(CurrentBar))                       //A new bar has just opened
	{
		CurrentBar = Time[0];
		
double MACD1 = iMACD(NULL,30,12,26,9,PRICE_CLOSE,MODE_MAIN,1),
       MACD2 = iMACD(NULL,30,12,26,9,PRICE_CLOSE,MODE_MAIN,2);
       
       
  if(MACD1 > MACD2 )        // Close Sell is True
     {
      Cls_S=true;                                          // Criterion for closing Sell
      Opn_B=true;                                          // Criterion for opening Buy      
     }
  
  if(MACD1 < MACD2 )        // Close Buy is True
  	  {
      Cls_B=true;                                         //  Criterion for closing Buy
      Opn_S=true;                                         //  Criterion for opening Sell
     }
     
//--------------------------------------------------------------- 6 --
   // Closing orders   
   while(true)                                  // Loop of closing orders 
     {
      if (Tip==0 && Cls_B==true)                // Order Buy is opened
        {                                       // and there is criterion to close
         Alert("Attempt to close Buy ",Ticket," of ",Symb," . Waiting for response..");
         RefreshRates();                        // Refresh rates
         Ans=OrderClose(Ticket,Lot,Bid,2);      // Closing Buy
         if (Ans==true)                         // Success :)
           {
            Alert ("Closed order Buy ",Ticket," of ",Symb);
            break;                              // Exit closing loop
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors  
            continue;                           // Retrying         
         return;                                // Exit start()
        }

      if (Tip==1 && Cls_S==true)                // Order Sell is opened..        
        {                                       // and there is criterion to close
         Alert("Attempt to close Sell ",Ticket," of ",Symb," . Waiting for response..");
         RefreshRates();                        // Refresh rates
         Ans=OrderClose(Ticket,Lot,Ask,2);      // Closing Sell
         if (Ans==true)                         // Success :)
           {
            Alert ("Closed order Sell ",Ticket," of ",Symb);
            break;                              //Exit closing loop
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors
            continue;                           // Retrying
         return;                                // Exit start()
        }
      break;                                    // Exit while
     }
//--------------------------------------------------------------- 7 --
   // Order value
   RefreshRates();                              // Refresh rates
   Min_Lot=MarketInfo(Symb,MODE_MINLOT);        // Minimal number of lots    
   Free   =AccountFreeMargin();                 // Free margin   
   Balance=AccountBalance();                    // Account Balance
   One_Lot=MarketInfo(Symb,MODE_MARGINREQUIRED);// Price of 1 lot
   Step   =MarketInfo(Symb,MODE_LOTSTEP);       // Step is changed 

   if (Lots > 0)                                // If lots are set, 
      Lts =Lots;                                // work with them
   else                                         // % of account balance
      Lts=MathFloor(Balance*Prots/One_Lot/Step)*Step;// For opening 

   if(Lts < Min_Lot) Lts=Min_Lot;               // Not less than minimal
   
   if(Free-(Lts*One_Lot) <= Balance*0.3)        // Free Margin musn't less than 30% of balance after opening position
   {
      Alert(" Not enough money for ", Lts," lots of ",Symb);
      return;                                   // Exit start()
     }
//--------------------------------------------------------------- 8 --
   // Opening orders   
   while(true)                                   // Orders opening loop      
     {

      if (Total==0 && Opn_B==true && USA_S(Cur_time)==true) // No opened orders +
        {                                       // criterion for opening Buy
         RefreshRates();                        // Refresh Rates
         SL=Bid - New_Stop(StopLoss)*Point;     // Calculating SL of opened
         Alert("Attempt to open Buy of ",Symb," . Waiting for response..");
         Ticket=OrderSend(Symb,OP_BUY,Lts,Ask,SLIPPAGE,SL,0,COMMENTARY_LONG,0,0,LimeGreen);// Opening Buy
         if (Ticket > 0)                        // Success :)
           {
            Alert ("Opened order Buy ",Ticket," of ",Symb);
            return;                             // Exit start()
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors 
            continue;                           // Retrying         
         return;                                // Exit start()
        }
      if (Total==0 && Opn_S==true && USA_S(Cur_time)==true)  // No opened orders +
        {                                       // criterion for opening Sell
         RefreshRates();                        // Refresh Reates
         SL=Ask + New_Stop(StopLoss)*Point;     // Calculating SL of opened
         Alert("Attempt to open Sell of ",Symb," . Waiting for response..");
         Ticket=OrderSend(Symb,OP_SELL,Lts,Bid,SLIPPAGE,SL,0,COMMENTARY_SHORT,0,0,Red);//Opening Sell
         if (Ticket > 0)                        //  Success :)
           {
            Alert ("Opened order Sell ",Ticket," of ",Symb);
            return;                             // Exit start()
           }
         if (Fun_Error(GetLastError())==1)      // Processing errors
            continue;                           // Retrying         
         return;                                // Exit start()
        }
      break;                                    // Exit while
     }
    }                                           // Exit the new bar flag
//--------------------------------------------------------------- 9 --
   return;                                      // Exit start()
  }
  
  //+---------------------Custom Functions-----------------------------+

//-------------------------------------------------------------- 10 --
bool NewBar(datetime PreviousBar)  // This function return the value true if the current bar/candle was just formed
{
    if(PreviousBar < Time[0])
        return(true);
    else
        return(false);
}

//-------------------------------------------------------------- 11 --
int Fun_Error(int Error)                        // Function of processing errors  
  {
   switch(Error)
     {                                          // Not crucial errors          
      case  4: Alert("for ",Symb,". Trade server is busy. Trying once again..");
         Sleep(3000);                           // Simple solution 
         return(1);                             // Exit the function
      case 135:Alert("for ",Symb,". Price changed. Trying once again..");
         RefreshRates();                        // Refresh Rates
         return(1);                             // Exit the function
      case 136:Alert("for ",Symb,". No prices. Waiting for a new tick..");
         while(RefreshRates()==false)           // Till a new tick            
            Sleep(1);                           // Pause in the loop         
         return(1);                             // Exit the function
      case 137:Alert("for ",Symb,". Broker is busy. Trying once again..");
         Sleep(3000);                           // Simple solution
         return(1);                             // Exit the function
      case 146:Alert("for ",Symb,". Trading subsystem is busy. Trying once again..");
         Sleep(500);                            // Simple solution
         return(1);                             // Exit the function
         // Critical errors
      case  2: Alert("for ",Symb,". Common error.");
         return(0);                             // Exit the function
      case  5: Alert("for ",Symb,". Old terminal version.");
         Work=false;                            // Terminate operation  
         return(0);                             // Exit the function
      case 64: Alert("for ",Symb,". Account blocked.");
         Work=false;                            // Terminate operation         
         return(0);                             // Exit the function
      case 133:Alert("for ",Symb,". Trading forbidden.");
         return(0);                             // Exit the function
      case 134:Alert("for ",Symb,". Not enough money to execute operation.");
         return(0);                             // Exit the function
      default: Alert("for ",Symb,". Error occurred: ",Error); // Other variants   
         return(0);                             // Exit the function
     }
  }

//-------------------------------------------------------------- 12 --

int New_Stop(int Parametr)                      //  Checking stop levels  
  {
   int Min_Dist=MarketInfo(Symb,MODE_STOPLEVEL);// Minimal distance   
   if (Parametr<Min_Dist)                       // If less than allowed     
     {
      Parametr=Min_Dist;                        // Sett allowed
      Alert("Increased distance of stop level.");
     }
   return(Parametr);                            // Returning value  
  }

//-------------------------------------------------------------- 13 --
bool USA_S (double time)                       // Cheking USA session
  {
   
   if  (22.00 > time >= 14.00)                 // If in USA session
        return(true);                          // Flag for USA session
    else
        return(false);                                  
  }
//-------------------------------------------------------------- 14 --